@@ -6,7 +6,7 @@
  *  (the "License"); you may not use this file except in compliance with
  *  the License.  You may obtain a copy of the License at
  *
- *      http://www.apache.org/licenses/LICENSE-2.0
+ *      https://www.apache.org/licenses/LICENSE-2.0
  *
  *  Unless required by applicable law or agreed to in writing, software
  *  distributed under the License is distributed on an "AS IS" BASIS,
@@ -17,20 +17,18 @@
  */
 package org.apache.tools.ant;
 
-import java.text.ParsePosition;
 import java.util.ArrayList;
 import java.util.Collection;
 import java.util.Collections;
-import java.util.Enumeration;
 import java.util.HashSet;
 import java.util.Hashtable;
 import java.util.List;
+import java.util.Map;
 import java.util.Set;
 import java.util.Vector;
 
 import org.apache.tools.ant.property.GetProperty;
 import org.apache.tools.ant.property.NullReturn;
-import org.apache.tools.ant.property.ParseNextProperty;
 import org.apache.tools.ant.property.ParseProperties;
 import org.apache.tools.ant.property.PropertyExpander;
 
@@ -72,7 +70,7 @@ import org.apache.tools.ant.property.PropertyExpander;
  * parseProperties} inside the ParseProperties class which in turn
  * uses the {@link org.apache.tools.ant.property.PropertyExpander
  * PropertyExpander delegates} to find properties inside the string
- * and this class to expand the propertiy names found into the
+ * and this class to expand the property names found into the
  * corresponding values.</p>
  *
  * <p>When {@link #getProperty looking up a property value} this class
@@ -163,7 +161,19 @@ public class PropertyHelper implements GetProperty {
             String property, Object value, PropertyHelper propertyHelper);
     }
 
-    //TODO PropertyEnumerator Delegate type, would improve PropertySet
+    /**
+     * Obtains the names of all known properties.
+     *
+     * @since 1.10.9
+     */
+    public interface PropertyEnumerator extends Delegate {
+        /**
+         * Returns the names of all properties known to this delegate.
+         *
+         * @return the names of all properties known to this delegate.
+         */
+        Set<String> getPropertyNames();
+    }
 
     //  --------------------------------------------------------
     //
@@ -184,52 +194,41 @@ public class PropertyHelper implements GetProperty {
         }
     };
 
-    private static final PropertyExpander DEFAULT_EXPANDER = new PropertyExpander() {
-        public String parsePropertyName(
-            String s, ParsePosition pos, ParseNextProperty notUsed) {
+    private static final PropertyExpander DEFAULT_EXPANDER =
+        (s, pos, notUsed) -> {
             int index = pos.getIndex();
             //directly check near, triggering characters:
-            if (s.length() - index >= 3
-                    && '$' == s.charAt(index) && '{' == s.charAt(index + 1)) {
+            if (s.length() - index >= 3 && '$' == s.charAt(index)
+                && '{' == s.charAt(index + 1)) {
                 int start = index + 2;
                 //defer to String.indexOf() for protracted check:
                 int end = s.indexOf('}', start);
                 if (end < 0) {
-                    throw new BuildException("Syntax error in property: "
-                            + s.substring(index));
+                    throw new BuildException(
+                        "Syntax error in property: " + s.substring(index));
                 }
                 pos.setIndex(end + 1);
-                return start == end ? "" :  s.substring(start, end);
+                return start == end ? "" : s.substring(start, end);
             }
             return null;
-        }
-    };
+        };
 
     /** dummy */
-    private static final PropertyExpander SKIP_DOUBLE_DOLLAR
-        = new PropertyExpander() {
-            // CheckStyle:LineLengthCheck OFF see too long
-            /**
-             * {@inheritDoc}
-             * @see org.apache.tools.ant.property.PropertyExpander#parsePropertyName(java.lang.String, java.text.ParsePosition, org.apache.tools.ant.PropertyHelper)
-             */
-            // CheckStyle:LineLengthCheck ON
-            public String parsePropertyName(
-                String s, ParsePosition pos, ParseNextProperty notUsed) {
-                int index = pos.getIndex();
-                if (s.length() - index >= 2) {
-                    /* check for $$; if found, advance by one--
-                     * this expander is at the bottom of the stack
-                     * and will thus be the last consulted,
-                     * so the next thing that ParseProperties will do
-                     * is advance the parse position beyond the second $
-                     */
-                    if ('$' == s.charAt(index) && '$' == s.charAt(++index)) {
-                        pos.setIndex(index);
-                    }
+    private static final PropertyExpander SKIP_DOUBLE_DOLLAR =
+        (s, pos, notUsed) -> {
+            int index = pos.getIndex();
+            if (s.length() - index >= 2) {
+                /* check for $$; if found, advance by one--
+                 * this expander is at the bottom of the stack
+                 * and will thus be the last consulted,
+                 * so the next thing that ParseProperties will do
+                 * is advance the parse position beyond the second $
+                 */
+                if ('$' == s.charAt(index) && '$' == s.charAt(++index)) {
+                    pos.setIndex(index);
                 }
-                return null;
             }
+            return null;
         };
 
     /**
@@ -248,24 +247,24 @@ public class PropertyHelper implements GetProperty {
 
     private Project project;
     private PropertyHelper next;
-    private final Hashtable<Class<? extends Delegate>, List<Delegate>> delegates = new Hashtable<Class<? extends Delegate>, List<Delegate>>();
+    private final Hashtable<Class<? extends Delegate>, List<Delegate>> delegates = new Hashtable<>();
 
     /** Project properties map (usually String to String). */
-    private Hashtable<String, Object> properties = new Hashtable<String, Object>();
+    private final Hashtable<String, Object> properties = new Hashtable<>();
 
     /**
      * Map of "user" properties (as created in the Ant task, for example).
      * Note that these key/value pairs are also always put into the
      * project properties, so only the project properties need to be queried.
      */
-    private Hashtable<String, Object> userProperties = new Hashtable<String, Object>();
+    private final Hashtable<String, Object> userProperties = new Hashtable<>();
 
     /**
      * Map of inherited "user" properties - that are those "user"
      * properties that have been created by tasks and not been set
      * from the command line or a GUI tool.
      */
-    private Hashtable<String, Object> inheritedProperties = new Hashtable<String, Object>();
+    private final Hashtable<String, Object> inheritedProperties = new Hashtable<>();
 
     /**
      * Default constructor.
@@ -360,6 +359,7 @@ public class PropertyHelper implements GetProperty {
      * @param next the next property helper in the chain.
      * @deprecated use the delegate mechanism instead
      */
+    @Deprecated
     public void setNext(PropertyHelper next) {
         this.next = next;
     }
@@ -375,6 +375,7 @@ public class PropertyHelper implements GetProperty {
      * @return the next property helper.
      * @deprecated use the delegate mechanism instead
      */
+    @Deprecated
     public PropertyHelper getNext() {
         return next;
     }
@@ -392,8 +393,7 @@ public class PropertyHelper implements GetProperty {
     public static synchronized PropertyHelper getPropertyHelper(Project project) {
         PropertyHelper helper = null;
         if (project != null) {
-            helper = (PropertyHelper) project.getReference(MagicNames
-                                                           .REFID_PROPERTY_HELPER);
+            helper = project.getReference(MagicNames.REFID_PROPERTY_HELPER);
         }
         if (helper != null) {
             return helper;
@@ -445,16 +445,14 @@ public class PropertyHelper implements GetProperty {
      *    has a good reason not to).
      * @deprecated PropertyHelper chaining is deprecated.
      */
+    @Deprecated
     public boolean setPropertyHook(String ns, String name,
                                    Object value,
                                    boolean inherited, boolean user,
                                    boolean isNew) {
         if (getNext() != null) {
-            boolean subst = getNext().setPropertyHook(ns, name, value, inherited, user, isNew);
             // If next has handled the property
-            if (subst) {
-                return true;
-            }
+            return getNext().setPropertyHook(ns, name, value, inherited, user, isNew);
         }
         return false;
     }
@@ -472,6 +470,7 @@ public class PropertyHelper implements GetProperty {
      * @return The property, if returned by a hook, or null if none.
      * @deprecated PropertyHelper chaining is deprecated.
      */
+    @Deprecated
     public Object getPropertyHook(String ns, String name, boolean user) {
         if (getNext() != null) {
             Object o = getNext().getPropertyHook(ns, name, user);
@@ -519,6 +518,7 @@ public class PropertyHelper implements GetProperty {
      *                           <code>}</code>
      * @deprecated use the other mechanisms of this class instead
      */
+    @Deprecated
     public void parsePropertyString(String value, Vector<String> fragments,
                                     Vector<String> propertyRefs) throws BuildException {
         parsePropertyStringDefault(value, fragments, propertyRefs);
@@ -618,6 +618,7 @@ public class PropertyHelper implements GetProperty {
      * @return true if the property is set.
      * @deprecated namespaces are unnecessary.
      */
+    @Deprecated
     public boolean setProperty(String ns, String name, Object value, boolean verbose) {
         return setProperty(name, value, verbose);
     }
@@ -675,6 +676,7 @@ public class PropertyHelper implements GetProperty {
      * @since Ant 1.6
      * @deprecated namespaces are unnecessary.
      */
+    @Deprecated
     public void setNewProperty(String ns, String name, Object value) {
         setNewProperty(name, value);
     }
@@ -726,6 +728,7 @@ public class PropertyHelper implements GetProperty {
      *              Must not be <code>null</code>.
      * @deprecated namespaces are unnecessary.
      */
+    @Deprecated
     public void setUserProperty(String ns, String name, Object value) {
         setUserProperty(name, value);
     }
@@ -768,6 +771,7 @@ public class PropertyHelper implements GetProperty {
      *              Must not be <code>null</code>.
      * @deprecated namespaces are unnecessary.
      */
+    @Deprecated
     public void setInheritedProperty(String ns, String name, Object value) {
         setInheritedProperty(name, value);
     }
@@ -814,6 +818,7 @@ public class PropertyHelper implements GetProperty {
      *         or if a <code>null</code> name is provided.
      * @deprecated namespaces are unnecessary.
      */
+    @Deprecated
     public Object getProperty(String ns, String name) {
         return getProperty(name);
     }
@@ -849,6 +854,18 @@ public class PropertyHelper implements GetProperty {
         return properties.get(name);
     }
 
+    /**
+     * Returns the names of all known properties.
+     * @since 1.10.9
+     * @return the names of all known properties.
+     */
+    public Set<String> getPropertyNames() {
+        final Set<String> names = new HashSet<>(getProperties().keySet());
+        getDelegates(PropertyEnumerator.class)
+            .forEach(e -> names.addAll(e.getPropertyNames()));
+        return Collections.unmodifiableSet(names);
+    }
+
     /**
      * Returns the value of a user property, if it is set.
      *
@@ -862,6 +879,7 @@ public class PropertyHelper implements GetProperty {
      *         or if a <code>null</code> name is provided.
      * @deprecated namespaces are unnecessary.
      */
+    @Deprecated
     public Object getUserProperty(String ns, String name) {
         return getUserProperty(name);
     }
@@ -900,7 +918,7 @@ public class PropertyHelper implements GetProperty {
     public Hashtable<String, Object> getProperties() {
         //avoid concurrent modification:
         synchronized (properties) {
-            return new Hashtable<String, Object>(properties);
+            return new Hashtable<>(properties);
         }
         // There is a better way to save the context. This shouldn't
         // delegate to next, it's for backward compatibility only.
@@ -917,7 +935,7 @@ public class PropertyHelper implements GetProperty {
     public Hashtable<String, Object> getUserProperties() {
         //avoid concurrent modification:
         synchronized (userProperties) {
-            return new Hashtable<String, Object>(userProperties);
+            return new Hashtable<>(userProperties);
         }
     }
 
@@ -932,7 +950,7 @@ public class PropertyHelper implements GetProperty {
     public Hashtable<String, Object> getInheritedProperties() {
         //avoid concurrent modification:
         synchronized (inheritedProperties) {
-            return new Hashtable<String, Object>(inheritedProperties);
+            return new Hashtable<>(inheritedProperties);
         }
     }
 
@@ -980,14 +998,11 @@ public class PropertyHelper implements GetProperty {
     public void copyInheritedProperties(Project other) {
         //avoid concurrent modification:
         synchronized (inheritedProperties) {
-            Enumeration<String> e = inheritedProperties.keys();
-            while (e.hasMoreElements()) {
-                String arg = e.nextElement().toString();
-                if (other.getUserProperty(arg) != null) {
-                    continue;
+            for (Map.Entry<String, Object> entry : inheritedProperties.entrySet()) {
+                String arg = entry.getKey();
+                if (other.getUserProperty(arg) == null) {
+                    other.setInheritedProperty(arg, entry.getValue().toString());
                 }
-                Object value = inheritedProperties.get(arg);
-                other.setInheritedProperty(arg, value.toString());
             }
         }
     }
@@ -1010,14 +1025,11 @@ public class PropertyHelper implements GetProperty {
     public void copyUserProperties(Project other) {
         //avoid concurrent modification:
         synchronized (userProperties) {
-            Enumeration<String> e = userProperties.keys();
-            while (e.hasMoreElements()) {
-                Object arg = e.nextElement();
-                if (inheritedProperties.containsKey(arg)) {
-                    continue;
+            for (Map.Entry<String, Object> entry : userProperties.entrySet()) {
+                String arg = entry.getKey();
+                if (!inheritedProperties.containsKey(arg)) {
+                    other.setUserProperty(arg, entry.getValue().toString());
                 }
-                Object value = userProperties.get(arg);
-                other.setUserProperty(arg.toString(), value.toString());
             }
         }
     }
@@ -1026,7 +1038,7 @@ public class PropertyHelper implements GetProperty {
     // Moved from ProjectHelper. You can override the static method -
     // this is used for backward compatibility (for code that calls
     // the parse method in ProjectHelper).
-
+    
     /**
      * Default parsing method. It is here only to support backward compatibility
      * for the static ProjectHelper.parsePropertyString().
@@ -1036,7 +1048,7 @@ public class PropertyHelper implements GetProperty {
         int prev = 0;
         int pos;
         //search for the next instance of $ from the 'prev' position
-        while ((pos = value.indexOf("$", prev)) >= 0) {
+        while ((pos = value.indexOf('$', prev)) >= 0) {
 
             //if there was any text before this, add it as a fragment
             //TODO, this check could be modified to go if pos>prev;
@@ -1060,12 +1072,11 @@ public class PropertyHelper implements GetProperty {
                 if (value.charAt(pos + 1) == '$') {
                     //backwards compatibility two $ map to one mode
                     fragments.addElement("$");
-                    prev = pos + 2;
                 } else {
                     //new behaviour: $X maps to $X for all values of X!='$'
                     fragments.addElement(value.substring(pos, pos + 2));
-                    prev = pos + 2;
                 }
+                prev = pos + 2;
             } else {
                 //property found, extract its name or bail on a typo
                 int endName = value.indexOf('}', pos);
@@ -1096,10 +1107,10 @@ public class PropertyHelper implements GetProperty {
             for (Class<? extends Delegate> key : getDelegateInterfaces(delegate)) {
                 List<Delegate> list = delegates.get(key);
                 if (list == null) {
-                    list = new ArrayList<Delegate>();
+                    list = new ArrayList<>();
                 } else {
                     //copy on write, top priority
-                    list = new ArrayList<Delegate>(list);
+                    list = new ArrayList<>(list);
                     list.remove(delegate);
                 }
                 list.add(0, delegate);
@@ -1111,6 +1122,7 @@ public class PropertyHelper implements GetProperty {
     /**
      * Get the Collection of delegates of the specified type.
      *
+     * @param <D> desired type.
      * @param type
      *            delegate type.
      * @return Collection.
@@ -1119,7 +1131,7 @@ public class PropertyHelper implements GetProperty {
     protected <D extends Delegate> List<D> getDelegates(Class<D> type) {
         @SuppressWarnings("unchecked")
         final List<D> result = (List<D>) delegates.get(type);
-        return result == null ? Collections.<D> emptyList() : result;
+        return result == null ? Collections.emptyList() : result;
     }
 
     /**
@@ -1128,16 +1140,14 @@ public class PropertyHelper implements GetProperty {
      * @return Set&lt;Class&gt;
      * @since Ant 1.8.0
      */
+    @SuppressWarnings("unchecked")
     protected static Set<Class<? extends Delegate>> getDelegateInterfaces(Delegate d) {
-        final HashSet<Class<? extends Delegate>> result = new HashSet<Class<? extends Delegate>>();
+        final HashSet<Class<? extends Delegate>> result = new HashSet<>();
         Class<?> c = d.getClass();
         while (c != null) {
-            Class<?>[] ifs = c.getInterfaces();
-            for (int i = 0; i < ifs.length; i++) {
-                if (Delegate.class.isAssignableFrom(ifs[i])) {
-                    @SuppressWarnings("unchecked")
-                    final Class<? extends Delegate> delegateInterface = (Class<? extends Delegate>) ifs[i];
-                    result.add(delegateInterface);
+            for (Class<?> ifc : c.getInterfaces()) {
+                if (Delegate.class.isAssignableFrom(ifc)) {
+                    result.add((Class<? extends Delegate>) ifc);
                 }
             }
             c = c.getSuperclass();
@@ -1149,6 +1159,8 @@ public class PropertyHelper implements GetProperty {
     /**
      * If the given object can be interpreted as a true/false value,
      * turn it into a matching Boolean - otherwise return null.
+     * @param value Object
+     * @return Boolean
      * @since Ant 1.8.0
      */
     public static Boolean toBoolean(Object value) {
@@ -1172,23 +1184,26 @@ public class PropertyHelper implements GetProperty {
     /**
      * Returns true if the object is null or an empty string.
      *
+     * @param value Object
+     * @return boolean
      * @since Ant 1.8.0
      */
     private static boolean nullOrEmpty(Object value) {
         return value == null || "".equals(value);
-
     }
 
     /**
      * Returns true if the value can be interpreted as a true value or
      * cannot be interpreted as a false value and a property of the
      * value's name exists.
+     * @param value Object
+     * @return boolean
      * @since Ant 1.8.0
      */
     private boolean evalAsBooleanOrPropertyName(Object value) {
         Boolean b = toBoolean(value);
         if (b != null) {
-            return b.booleanValue();
+            return b;
         }
         return getProperty(String.valueOf(value)) != null;
     }
@@ -1197,6 +1212,8 @@ public class PropertyHelper implements GetProperty {
      * Returns true if the value is null or an empty string, can be
      * interpreted as a true value or cannot be interpreted as a false
      * value and a property of the value's name exists.
+     * @param value Object
+     * @return boolean
      * @since Ant 1.8.0
      */
     public boolean testIfCondition(Object value) {
@@ -1207,6 +1224,8 @@ public class PropertyHelper implements GetProperty {
      * Returns true if the value is null or an empty string, can be
      * interpreted as a false value or cannot be interpreted as a true
      * value and a property of the value's name doesn't exist.
+     * @param value Object
+     * @return boolean
      * @since Ant 1.8.0
      */
     public boolean testUnlessCondition(Object value) {
