@@ -6,7 +6,7 @@
  *  (the "License"); you may not use this file except in compliance with
  *  the License.  You may obtain a copy of the License at
  *
- *      http://www.apache.org/licenses/LICENSE-2.0
+ *      https://www.apache.org/licenses/LICENSE-2.0
  *
  *  Unless required by applicable law or agreed to in writing, software
  *  distributed under the License is distributed on an "AS IS" BASIS,
@@ -26,10 +26,9 @@ import java.util.Hashtable;
 import java.util.LinkedHashMap;
 import java.util.List;
 import java.util.Map;
-import java.util.Iterator;
 
-import org.apache.tools.ant.util.CollectionUtils;
-import org.apache.tools.ant.taskdefs.MacroDef;
+import org.apache.tools.ant.attribute.EnableAttribute;
+import org.apache.tools.ant.taskdefs.MacroDef.Attribute;
 import org.apache.tools.ant.taskdefs.MacroInstance;
 import org.xml.sax.AttributeList;
 import org.xml.sax.helpers.AttributeListImpl;
@@ -44,32 +43,31 @@ public class RuntimeConfigurable implements Serializable {
     /** Serialization version */
     private static final long serialVersionUID = 1L;
 
-    /** Empty Hashtable. */
-    private static final Hashtable EMPTY_HASHTABLE = new Hashtable(0);
-
     /** Name of the element to configure. */
     private String elementTag = null;
 
     /** List of child element wrappers. */
-    private List/*<RuntimeConfigurable>*/ children = null;
+    // picking ArrayList rather than List as arrayList is Serializable
+    private List<RuntimeConfigurable> children = null;
 
     /** The element to configure. It is only used during
      * maybeConfigure.
      */
     private transient Object wrappedObject = null;
 
-    /** the creator used to make the wrapped object */
-    private transient IntrospectionHelper.Creator creator;
-
     /**
      * XML attributes for the element.
      * @deprecated since 1.6.x
      */
+    @Deprecated
     private transient AttributeList attributes;
 
+    // The following is set to true if any of the attributes are namespaced
+    private transient boolean namespacedAttribute = false;
+
     /** Attribute names and values. While the XML spec doesn't require
-     *  preserving the order ( AFAIK ), some ant tests do rely on the
-     *  exact order. 
+     *  preserving the order (AFAIK), some ant tests do rely on the
+     *  exact order.
      * The only exception to this order is the treatment of
      * refid. A number of datatypes check if refid is set
      * when other attributes are set. This check will not
@@ -77,7 +75,7 @@ public class RuntimeConfigurable implements Serializable {
      * the "refid" attribute, so now (ANT 1.7) the refid
      * attribute will be processed first.
      */
-    private LinkedHashMap/*<String, String>*/ attributeMap = null;
+    private LinkedHashMap<String, Object> attributeMap = null;
 
     /** Text appearing within the element. */
     private StringBuffer characters = null;
@@ -116,6 +114,110 @@ public class RuntimeConfigurable implements Serializable {
         proxyConfigured = false;
     }
 
+    private static class EnableAttributeConsumer {
+        @SuppressWarnings("unused")
+        public void add(EnableAttribute b) {
+            // Ignore
+        }
+    }
+
+    /**
+     * contains the attribute component name and boolean restricted set to true when
+     * the attribute is in one of the name spaces managed by ant (if and unless currently)
+     * @since Ant 1.9.3
+     */
+    private static class AttributeComponentInformation {
+        String componentName;
+        boolean restricted;
+
+        private AttributeComponentInformation(String componentName, boolean restricted) {
+            this.componentName = componentName;
+            this.restricted = restricted;
+        }
+
+        public String getComponentName() {
+            return componentName;
+        }
+
+        public boolean isRestricted() {
+            return restricted;
+        }
+    }
+
+    /**
+     *
+     * @param name    the name of the attribute.
+     * @param componentHelper current component helper
+     * @return AttributeComponentInformation instance
+     */
+    private AttributeComponentInformation isRestrictedAttribute(String name, ComponentHelper componentHelper) {
+        if (!name.contains(":")) {
+            return new AttributeComponentInformation(null, false);
+        }
+        String componentName = attrToComponent(name);
+        String ns = ProjectHelper.extractUriFromComponentName(componentName);
+        if (componentHelper.getRestrictedDefinitions(
+                ProjectHelper.nsToComponentName(ns)) == null) {
+            return new AttributeComponentInformation(null, false);
+        }
+        return new AttributeComponentInformation(componentName, true);
+    }
+
+    /**
+     * Check if an UE is enabled.
+     * This looks tru the attributes and checks if there
+     * are any Ant attributes, and if so, the method calls the
+     * isEnabled() method on them.
+     * @param owner the UE that owns this RC.
+     * @return true if enabled, false if any of the ant attributes return
+     *              false.
+     * @since 1.9.1
+     */
+    @SuppressWarnings("deprecation")
+    public boolean isEnabled(UnknownElement owner) {
+        if (!namespacedAttribute) {
+            return true;
+        }
+        ComponentHelper componentHelper = ComponentHelper
+            .getComponentHelper(owner.getProject());
+
+        IntrospectionHelper ih = IntrospectionHelper.getHelper(owner.getProject(),
+                EnableAttributeConsumer.class);
+        for (Map.Entry<String, Object> entry : attributeMap.entrySet()) {
+            AttributeComponentInformation attributeComponentInformation
+                    = isRestrictedAttribute(entry.getKey(), componentHelper);
+            if (!attributeComponentInformation.isRestricted())  {
+                continue;
+            }
+            String value = (String) entry.getValue();
+            EnableAttribute enable = null;
+            try {
+                enable = (EnableAttribute)
+                    ih.createElement(
+                        owner.getProject(), new EnableAttributeConsumer(),
+                        attributeComponentInformation.getComponentName());
+            } catch (BuildException ex) {
+                throw new BuildException(
+                    "Unsupported attribute " + attributeComponentInformation.getComponentName());
+            }
+            if (enable == null) {
+                continue;
+            }
+            value = owner.getProject().replaceProperties(value); // FixMe: need to make config
+            if (!enable.isEnabled(owner, value)) {
+                return false;
+            }
+        }
+        return true;
+    }
+
+    private String attrToComponent(String a) {
+        // need to remove the prefix
+        int p1 = a.lastIndexOf(':');
+        int p2 = a.lastIndexOf(':', p1 - 1);
+        return a.substring(0, p2) + a.substring(p1);
+    }
+
     /**
      * Sets the creator of the element to be configured
      * used to store the element in the parent.
@@ -123,7 +225,6 @@ public class RuntimeConfigurable implements Serializable {
      * @param creator the creator object.
      */
     synchronized void setCreator(IntrospectionHelper.Creator creator) {
-        this.creator = creator;
     }
 
     /**
@@ -163,10 +264,11 @@ public class RuntimeConfigurable implements Serializable {
     /**
      * Sets the attributes for the wrapped element.
      *
-     * @deprecated since 1.6.x.
      * @param attributes List of attributes defined in the XML for this
      *                   element. May be <code>null</code>.
+     * @deprecated since 1.6.x.
      */
+    @Deprecated
     public synchronized void setAttributes(AttributeList attributes) {
         this.attributes = new AttributeListImpl(attributes);
         for (int i = 0; i < attributes.getLength(); i++) {
@@ -181,22 +283,36 @@ public class RuntimeConfigurable implements Serializable {
      * @param value the attribute's value.
      */
     public synchronized void setAttribute(String name, String value) {
+        if (name.contains(":")) {
+            namespacedAttribute = true;
+        }
+        setAttribute(name, (Object) value);
+    }
+
+    /**
+     * Set an attribute to a given value.
+     *
+     * @param name the name of the attribute.
+     * @param value the attribute's value.
+     * @since 1.9
+     */
+    public synchronized void setAttribute(String name, Object value) {
         if (name.equalsIgnoreCase(ProjectHelper.ANT_TYPE)) {
-            this.polyType = value;
+            this.polyType = value == null ? null : value.toString();
         } else {
             if (attributeMap == null) {
-                attributeMap = new LinkedHashMap();
+                attributeMap = new LinkedHashMap<>();
             }
-            if (name.equalsIgnoreCase("refid") && !attributeMap.isEmpty()) {
-                LinkedHashMap newAttributeMap = new LinkedHashMap();
+            if ("refid".equalsIgnoreCase(name) && !attributeMap.isEmpty()) {
+                LinkedHashMap<String, Object> newAttributeMap = new LinkedHashMap<>();
                 newAttributeMap.put(name, value);
                 newAttributeMap.putAll(attributeMap);
                 attributeMap = newAttributeMap;
             } else {
                 attributeMap.put(name, value);
             }
-            if (name.equals("id")) {
-                this.id = value;
+            if ("id".equals(name)) {
+                this.id = value == null ? null : value.toString();
             }
         }
     }
@@ -215,18 +331,18 @@ public class RuntimeConfigurable implements Serializable {
      * @return Attribute name to attribute value map.
      * @since Ant 1.6
      */
-    public synchronized Hashtable getAttributeMap() {
-        return (attributeMap == null)
-            ? EMPTY_HASHTABLE : new Hashtable(attributeMap);
+    public synchronized Hashtable<String, Object> getAttributeMap() {
+        return new Hashtable<>(attributeMap == null ? Collections.emptyMap() : attributeMap);
     }
 
     /**
      * Returns the list of attributes for the wrapped element.
      *
-     * @deprecated Deprecated since Ant 1.6 in favor of {@link #getAttributeMap}.
      * @return An AttributeList representing the attributes defined in the
      *         XML for this element. May be <code>null</code>.
+     * @deprecated Deprecated since Ant 1.6 in favor of {@link #getAttributeMap}.
      */
+    @Deprecated
     public synchronized AttributeList getAttributes() {
         return attributes;
     }
@@ -238,7 +354,7 @@ public class RuntimeConfigurable implements Serializable {
      *              Must not be <code>null</code>.
      */
     public synchronized void addChild(RuntimeConfigurable child) {
-        children = (children == null) ? new ArrayList() : children;
+        children = (children == null) ? new ArrayList<>() : children;
         children.add(child);
     }
 
@@ -251,7 +367,7 @@ public class RuntimeConfigurable implements Serializable {
      *         list.
      */
     synchronized RuntimeConfigurable getChild(int index) {
-        return (RuntimeConfigurable) children.get(index);
+        return children.get(index);
     }
 
     /**
@@ -259,8 +375,8 @@ public class RuntimeConfigurable implements Serializable {
      * @return an enumeration of the child wrappers.
      * @since Ant 1.6
      */
-    public synchronized Enumeration getChildren() {
-        return (children == null) ? new CollectionUtils.EmptyEnumeration()
+    public synchronized Enumeration<RuntimeConfigurable> getChildren() {
+        return (children == null) ? Collections.emptyEnumeration()
             : Collections.enumeration(children);
     }
 
@@ -271,7 +387,7 @@ public class RuntimeConfigurable implements Serializable {
      *        Should not be <code>null</code>.
      */
     public synchronized void addText(String data) {
-        if (data.length() == 0) {
+        if (data.isEmpty()) {
             return;
         }
         characters = (characters == null)
@@ -297,7 +413,7 @@ public class RuntimeConfigurable implements Serializable {
 
     /**
      * Get the text content of this element. Various text chunks are
-     * concatenated, there is no way ( currently ) of keeping track of
+     * concatenated, there is no way (currently) of keeping track of
      * multiple fragments.
      *
      * @return the text content of this element.
@@ -331,9 +447,10 @@ public class RuntimeConfigurable implements Serializable {
      * and then each child is configured and added. Each time the
      * wrapper is configured, the attributes and text for it are
      * reset.
-     *
+     * <p>
      * If the element has an <code>id</code> attribute, a reference
      * is added to the project as well.
+     * </p>
      *
      * @param p The project containing the wrapped element.
      *          Must not be <code>null</code>.
@@ -350,18 +467,18 @@ public class RuntimeConfigurable implements Serializable {
      * Configures the wrapped element.  The attributes and text for
      * the wrapped element are configured.  Each time the wrapper is
      * configured, the attributes and text for it are reset.
-     *
+     * <p>
      * If the element has an <code>id</code> attribute, a reference
      * is added to the project as well.
+     * </p>
      *
      * @param p The project containing the wrapped element.
      *          Must not be <code>null</code>.
      *
      * @param configureChildren ignored.
-
      *
      * @exception BuildException if the configuration fails, for instance due
-     *            to invalid attributes , or text being added to
+     *            to invalid attributes, or text being added to
      *            an element which doesn't accept it.
      */
     public synchronized void maybeConfigure(Project p, boolean configureChildren)
@@ -371,26 +488,34 @@ public class RuntimeConfigurable implements Serializable {
             return;
         }
 
-        // Configure the object
-        Object target = (wrappedObject instanceof TypeAdapter)
-            ? ((TypeAdapter) wrappedObject).getProxy() : wrappedObject;
-
-        IntrospectionHelper ih =
-            IntrospectionHelper.getHelper(p, target.getClass());
-
         if (attributeMap != null) {
-            for (Iterator iter = attributeMap.entrySet().iterator(); iter.hasNext();) {
-                Map.Entry entry = (Map.Entry) iter.next();
-                String name = (String) entry.getKey();
-                String value = (String) entry.getValue();
+            // Configure the object
+            Object target = (wrappedObject instanceof TypeAdapter)
+                    ? ((TypeAdapter) wrappedObject).getProxy() : wrappedObject;
+                    
+            IntrospectionHelper ih =
+                    IntrospectionHelper.getHelper(p, target.getClass());
+            ComponentHelper componentHelper = ComponentHelper.getComponentHelper(p);
 
+            for (Map.Entry<String, Object> entry : attributeMap.entrySet()) {
+                String name = entry.getKey();
+                // skip restricted attributes such as if:set
+                AttributeComponentInformation attributeComponentInformation = isRestrictedAttribute(name, componentHelper);
+                if (attributeComponentInformation.isRestricted())  {
+                    continue;
+                }
+                Object value = entry.getValue();
                 // reflect these into the target, defer for
                 // MacroInstance where properties are expanded for the
                 // nested sequential
-                Object attrValue = PropertyHelper.getPropertyHelper(p).parseProperties(value);
+                Object attrValue;
+                if (value instanceof Evaluable<?>) {
+                    attrValue = ((Evaluable<?>) value).eval();
+                } else {
+                    attrValue = PropertyHelper.getPropertyHelper(p).parseProperties(value.toString());
+                }
                 if (target instanceof MacroInstance) {
-                    for (Iterator attrs = ((MacroInstance) target).getMacroDef().getAttributes().iterator(); attrs.hasNext();) {
-                        MacroDef.Attribute attr = (MacroDef.Attribute) attrs.next();
+                    for (Attribute attr : ((MacroInstance) target).getMacroDef().getAttributes()) {
                         if (attr.getName().equals(name)) {
                             if (!attr.isDoubleExpanding()) {
                                 attrValue = value;
@@ -403,23 +528,22 @@ public class RuntimeConfigurable implements Serializable {
                     ih.setAttribute(p, target, name, attrValue);
                 } catch (UnsupportedAttributeException be) {
                     // id attribute must be set externally
-                    if (name.equals("id")) {
-                        // Do nothing
-                    } else if (getElementTag() == null) {
-                        throw be;
-                    } else {
-                        throw new BuildException(
-                            getElementTag() + " doesn't support the \""
-                            + be.getAttribute() + "\" attribute", be);
+                    if (!"id".equals(name)) {
+                        if (getElementTag() == null) {
+                            throw be;
+                        } else {
+                            throw new BuildException(
+                                getElementTag() + " doesn't support the \""
+                                + be.getAttribute() + "\" attribute", be);
+                        }
                     }
                 } catch (BuildException be) {
-                    if (name.equals("id")) {
-                        // Assume that this is an not supported attribute type
-                        // thrown for example by a dymanic attribute task
-                        // Do nothing
-                    } else {
+                    if (!"id".equals(name)) {
                         throw be;
                     }
+                    // Assume that this is an not supported attribute type
+                    // thrown for example by a dynamic attribute task --
+                    // do nothing
                 }
             }
         }
@@ -453,8 +577,7 @@ public class RuntimeConfigurable implements Serializable {
     public void applyPreSet(RuntimeConfigurable r) {
         // Attributes
         if (r.attributeMap != null) {
-            for (Iterator i = r.attributeMap.keySet().iterator(); i.hasNext();) {
-                String name = (String) i.next();
+            for (String name : r.attributeMap.keySet()) {
                 if (attributeMap == null || attributeMap.get(name) == null) {
                     setAttribute(name, (String) r.attributeMap.get(name));
                 }
@@ -466,8 +589,7 @@ public class RuntimeConfigurable implements Serializable {
 
         // Children (this is a shadow of UnknownElement#children)
         if (r.children != null) {
-            List newChildren = new ArrayList();
-            newChildren.addAll(r.children);
+            List<RuntimeConfigurable> newChildren = new ArrayList<>(r.children);
             if (children != null) {
                 newChildren.addAll(children);
             }
@@ -477,7 +599,7 @@ public class RuntimeConfigurable implements Serializable {
         // Text
         if (r.characters != null) {
             if (characters == null
-                || characters.toString().trim().length() == 0) {
+                || characters.toString().trim().isEmpty()) {
                 characters = new StringBuffer(r.characters.toString());
             }
         }
