@@ -6,7 +6,7 @@
  *  (the "License"); you may not use this file except in compliance with
  *  the License.  You may obtain a copy of the License at
  *
- *      http://www.apache.org/licenses/LICENSE-2.0
+ *      https://www.apache.org/licenses/LICENSE-2.0
  *
  *  Unless required by applicable law or agreed to in writing, software
  *  distributed under the License is distributed on an "AS IS" BASIS,
@@ -32,11 +32,7 @@ public class ThreadedOutput extends TestCase {
     }
 
     public void testOutput() throws InterruptedException {
-        Thread t = new Thread(new Runnable() {
-                public void run() {
-                    System.out.println("foo");
-                }
-            });
+        Thread t = new Thread(() -> System.out.println("foo"));
         t.start();
         t.join();
     }
